#include <sys/exception.h>
#include <cdefBF533.h>
#include <stdio.h>
#include <ccblkfn.h>
#include "UART_Library.h"
#include "G168_UART_Interface.h"
#include <string.h>
#include "Queue.h"

Queue *cmdQueue;

EX_INTERRUPT_HANDLER(ISR_UART_Rx);

EX_INTERRUPT_HANDLER(ISR_UART_Rx)
{
	unsigned char inData;
	static char tempStr[YY_BUF_SIZE];
	static char *charPtr = tempStr;
		
	// read received value (also clears the interrupt)
	inData = *pUART_RBR;

	if (inData == 0x0d) { // 0x0d == ENTER
		//*charPtr = 0x00;  // 0x00 == EOF
		*charPtr = inData;
		enqueue(cmdQueue, tempStr);
		charPtr = tempStr;
		// confirm by sending the LF/CR combination to the UART
		UART_puts("\n\r");
		return;
	}
	else {
		*charPtr = inData;
		charPtr++;
		// confirm by sending the received value back via UART
		*pUART_THR = inData;
	}
	
}

/* Implementation of the Yacc buffer input function */
int pipe_yyinput(char buf[YY_BUF_SIZE], int max_size)
{
	int len;

	if (getQueueCount(cmdQueue) > 0) {
		dequeue(cmdQueue, buf);
		len = strlen(buf);
		return len;
	}

	else {
		yy_flush_buffer();
		return 0;
	}
}

/* Implementation of the Yacc buffer output function */
int pipe_yyoutput(char buf[], int len)
{
	UART_puts(buf);
	return 0;
}

void main() {
	int period, divisor, bitrate;
	char str[255];
	short temp;

	cmdQueue = createQueue();

	/* UART Initialization */  
	period = UART_detectAutobaud();

	// Apply formula divisor = period / (16 x 8 bits)
	divisor = period >> 7;
	UART_init(divisor);

	bitrate = SYSTEM_CLOCK/16/divisor;
	// \n\r is a CR, LF escape character combination
    sprintf(str, "ADSP-BF533 Blackfin is operating at %d baud\n\r", bitrate);
	UART_puts(str);
	
	/* Set up thr UART interrupts */
	*pILAT |= EVT_IVG10; // clear pending IVG10 interrupts
	*pSIC_IAR1 = P14_IVG(10);
	ssync();
	*pSIC_IMASK |= SIC_MASK(14);
	register_handler(ik_ivg10, ISR_UART_Rx); 

	temp = *pUART_RBR;
	temp = *pUART_LSR;
	temp = *pUART_IIR;
	// Setting the ETBEI bit automatically fires a TX interrupt request.
	*pUART_IER= ERBFI;
	ssync();
	
	while(1) {		
		// if the queue is not empty, parse the commands
		if (getQueueCount(cmdQueue) > 0) {
			yyparse();
		}
	}
}

